@@ -1,6 +1,6 @@
 
 /*
- * $Id: ui.c,v 1.39 2005/08/23 10:03:34 jheinonen Exp $
+ * $Id: ui.c,v 1.54 2005/10/05 11:03:36 jheinonen Exp $
  *
  * by JH <jheinonen@users.sourceforge.net>
  *
@@ -15,9 +15,11 @@
 #include <signal.h>
 #include <ctype.h>
 #include "abook.h"
+#include <assert.h>
 #include "ui.h"
 #include "edit.h"
 #include "database.h"
+#include "gettext.h"
 #include "list.h"
 #include "misc.h"
 #include "options.h"
@@ -141,10 +143,10 @@ init_ui()
 #endif
 	if( LINES < MIN_LINES || COLS < MIN_COLS ) {
 		clear(); refresh(); endwin();
-		fprintf(stderr, "Your terminal size is %dx%d\n", COLS, LINES);
-		fprintf(stderr, "Terminal is too small. Minium terminal size "
-				"for abook is "
-				"%dx%d\n", MIN_COLS, MIN_LINES);
+		fprintf(stderr, _("Your terminal size is %dx%d\n"), COLS, LINES);
+		fprintf(stderr, _("Terminal is too small. Minimum terminal "
+				"size for abook is "
+				"%dx%d\n"), MIN_COLS, MIN_LINES);
 		return 1;
 	}
 
@@ -174,7 +176,7 @@ close_ui()
 
 
 void
-headerline(char *str)
+headerline(const char *str)
 {
 	werase(top);
 
@@ -199,7 +201,7 @@ refresh_screen()
 	clear();
 
 	refresh_statusline();
-	headerline(MAIN_HELPLINE);
+	headerline(gettext(MAIN_HELPLINE));
 	list_headerline();
 
 	refresh_list();
@@ -207,7 +209,7 @@ refresh_screen()
 
 
 int
-statusline_msg(char *msg)
+statusline_msg(const char *msg)
 {
 	int c;
 
@@ -223,15 +225,88 @@ statusline_msg(char *msg)
 }
 
 void
-statusline_addstr(char *str)
+statusline_addstr(const char *str)
 {
 	mvwaddstr(bottom, 1, 0, str);
 	refresh();
 	wrefresh(bottom);
 }
 
+/* Same as statusline_addstr(), but hilight "<str>" sequences if the terminal
+ * supports it */
+static void
+statusline_addhlstr(const char *str)
+{
+#if defined(A_BOLD) && defined(A_NORMAL) && defined(A_DIM)
+	const char *p = str, *start = str;
+	char *tmp;
+	int pos = 0;
+
+	while(1) {
+		if(!*p || strchr("<>", *p)) {
+			if(p - start > 0) {
+				wattrset(bottom, (*p == '>') ? A_BOLD : A_NORMAL);
+				tmp = xstrndup(start, p - start);
+				mvwaddstr(bottom, 1, pos, tmp);
+				pos += strwidth(tmp);
+				free(tmp);
+			}
+			if(*p) {
+				start = p + 1;
+
+				/* show tag markers */
+				wattrset(bottom, A_DIM);
+				mvwaddch(bottom, 1, pos++, *p);
+			}
+		}
+
+		if(!*p) {
+			wattrset(bottom, A_NORMAL);
+			break;
+		}
+
+		p++;
+	}
+#else
+	mvwaddstr(bottom, 1, 0, str);
+#endif
+
+	refresh();
+	wrefresh(bottom);
+}
+
+int
+statusline_askchoice(const char *msg, const char *choices, short dflt)
+{
+	char *s;
+	int ch;
+
+	assert((dflt < 0) || (dflt > strlen(choices)));
+
+	if(dflt) {
+		s = strdup_printf("%s [%c]", msg, choices[dflt - 1]);
+		statusline_addhlstr(s);
+		free(s);
+	} else
+		statusline_addhlstr(msg);
+
+	while(1)
+	{
+		ch = tolower(getch());
+
+		if(ch == 7) /* ctrl+G */
+			return 0;
+
+		if(dflt && (ch == '\r')) /* default choice */
+			return dflt;
+
+		if((s = strchr(choices, ch)))
+			return (s - choices + 1);
+	}
+}
+
 char *
-ui_readline(char *prompt, char *s, int limit, bool use_completion)
+ui_readline(char *prompt, char *s, size_t limit, bool use_completion)
 {
 	int y, x;
 	char *ret;
@@ -240,10 +315,13 @@ ui_readline(char *prompt, char *s, int limit, bool use_completion)
 
 	getyx(bottom, y, x);
 
-	ret = abook_readline(bottom, y, x, s, limit, use_completion);
+	ret = abook_readline(bottom, y, x, s, use_completion);
 
-	if(ret)
+	if(ret) {
 		strtrim(ret);
+		if(strlen(ret) > limit && limit > 0)
+			ret[limit] = '\0';
+	}
 
 	return ret;
 }
@@ -252,25 +330,21 @@ int
 statusline_ask_boolean(char *msg, int def)
 {
 	int ret;
-	char *msg2 = strconcat(msg,  def ? " (Y/n)?" : " (y/N)?", NULL);
+	char *msg2 = strconcat(msg,  def ? _(" (Y/n)?") : _(" (y/N)?"), NULL);
+	char ch;
 
 	statusline_addstr(msg2);
 
 	free(msg2);
 
-	switch(tolower(getch())) {
-		case 'n':
-		case 'N':
-			ret = FALSE;
-			break;
-		case 'y':
-		case 'Y':
-			ret = TRUE;
-			break;
-		default:
-			ret = def;
-			break;
-	}
+	ch = tolower(getch());
+
+	if(ch == *(S_("keybinding for no|n")))
+		ret = FALSE;
+	else if(ch == *(S_("keybinding for yes|y")))
+		ret = TRUE;
+	else
+		ret = def;
 
 	clear_statusline();
 
@@ -334,16 +408,16 @@ display_help(int help)
 
 	helpw = newwin(LINES - 5, COLS - 6, 2, 3);
 	erase();
-	headerline("help");
+	headerline(_("help"));
 
 	for(i = 0; tbl[i] != NULL; i++) {
-		waddstr(helpw, tbl[i]);
+		waddstr(helpw, gettext(tbl[i]));
 		if( (!((i + 1) % (LINES - 8))) ||
 			(tbl[i + 1] == NULL) ) {
 			refresh();
 			wrefresh(helpw);
 			refresh_statusline();
-			if(statusline_msg("Press any key to continue...")
+			if(statusline_msg(_("Press any key to continue..."))
 					== 'q')
 				break;
 			wclear(helpw);
@@ -451,7 +525,7 @@ get_commands()
 
 			case 'p': ui_print_database(); break;
 
-			case 'u': launch_wwwbrowser(list_current_item());
+			case 'v': launch_wwwbrowser(list_current_item());
 				  refresh_screen();
 				  break;
 		}
@@ -464,7 +538,7 @@ ui_remove_items()
 	if(list_is_empty())
 		return;
 
-	if(statusline_ask_boolean("Remove selected item(s)", TRUE))
+	if(statusline_ask_boolean(_("Remove selected item(s)"), TRUE))
 		remove_selected_items();
 
 	clear_statusline();
@@ -474,7 +548,7 @@ ui_remove_items()
 void
 ui_clear_database()
 {
-	if(statusline_ask_boolean("Clear WHOLE database", FALSE)) {
+	if(statusline_ask_boolean(_("Clear WHOLE database"), FALSE)) {
 		close_database();
 		refresh_list();
 	}
@@ -496,12 +570,13 @@ ui_find(int next)
 		char *s;
 		s = ui_readline("/", findstr, MAX_FIELD_LEN - 1, 0);
 		strncpy(findstr, s, MAX_FIELD_LEN);
+		free(s);
 		refresh_screen();
 	}
 
 	if( (item = find_item(findstr, curitem + !!next, search_fields)) < 0 &&
 			(item = find_item(findstr, 0, search_fields)) >= 0)
-		statusline_addstr("Search hit bottom, continuing at top");
+		statusline_addstr(_("Search hit bottom, continuing at top"));
 
 	if(item >= 0) {
 		curitem = item;
@@ -513,7 +588,7 @@ ui_find(int next)
 void
 ui_print_number_of_items()
 {
-	char *str = mkstr("     " "|%3d/%3d", selected_items(), items);
+	char *str = strdup_printf("     " "|%3d/%3d", selected_items(), items);
 
 	mvaddstr(0, COLS-strlen(str), str);
 
@@ -523,10 +598,18 @@ ui_print_number_of_items()
 void
 ui_read_database()
 {
-	if(items > 0)
-		if(!statusline_ask_boolean("Your current data will be lost - "
-				"Press 'y' to continue", FALSE))
+	char *msg;
+
+	if(items > 0) {
+		msg = strdup_printf(_("Your current data will be lost - "
+				"Press '%c' to continue"),
+				*(S_("keybinding for yes|y")));
+		if(!statusline_ask_boolean(msg, FALSE)) {
+			free(msg);
 			return;
+		}
+		free(msg);
+	}
 
 	load_database(datafile);
 	refresh_list();
@@ -543,21 +626,19 @@ ui_print_database()
 	if(list_is_empty())
 		return;
 
-	statusline_addstr("Print All/Selected/Cancel (a/s/C)?");
-
-	switch(tolower(getch())) {
-		case 'a':
+	switch(statusline_askchoice(_("Print <a>ll, print <s>elected, or <c>ancel?"), S_("keybindings:all/selected/cancel|asc"), 3)) {
+		case 1:
 			mode = ENUM_ALL;
 			break;
-		case 's':
+		case 2:
 			if( !selected_items() ) {
-				statusline_msg("No selected items");
+				statusline_msg(_("No selected items"));
 				return;
 			}
 			mode = ENUM_SELECTED;
 			break;
 		default:
-			clear_statusline();
+			refresh_screen();
 			return;
 	}
 
@@ -577,7 +658,7 @@ ui_open_datafile()
 {
 	char *filename;
 
-	filename = ask_filename("File to open: ");
+	filename = ask_filename(_("File to open: "));
 
 	if(!filename || ! *filename) {
 		free(filename);
@@ -587,7 +668,7 @@ ui_open_datafile()
 
 	if(opt_get_bool(BOOL_AUTOSAVE))
 		save_database();
-	else if(statusline_ask_boolean("Save current database", FALSE))
+	else if(statusline_ask_boolean(_("Save current database"), FALSE))
 		save_database();
 
 	close_database();
@@ -595,7 +676,7 @@ ui_open_datafile()
 	load_database(filename);
 
 	if(items == 0) {
-		statusline_msg("Sorry, that specified file appears not to be a valid abook addressbook");
+		statusline_msg(_("Sorry, the specified file appears not to be a valid abook addressbook"));
 		load_database(datafile);
 	} else {
 		free(datafile);
