@@ -12,15 +12,16 @@ int		is_ui_initialized();
 void		ui_init_curses();
 int		init_ui();
 void		close_ui();
-void		headerline(char *str);
+void		headerline(const char *str);
 void            refresh_screen();
-int		statusline_msg(char *msg);
+int		statusline_msg(const char *msg);
+int 		statusline_askchoice(const char *msg, const char *choices, short dflt);
 char		*ask_filename(char *prompt);
 int		statusline_ask_boolean(char *msg, int def);
 void            clear_statusline();
 void		display_help(int help);
-void		statusline_addstr(char *str);
-char *		ui_readline(char *prompt, char *s, int limit, bool use_completion);
+void		statusline_addstr(const char *str);
+char *		ui_readline(char *prompt, char *s, size_t limit, bool use_completion);
 void		refresh_statusline();
 void		get_commands();
 void		ui_remove_items();
